@@ -7,9 +7,9 @@ pub use writer::PackedWriter;
 
 use std::io;
 
-use super::{BinarySource, DomainDecode, Embeddable, IOValue, IOValueDomainCodec, NestedValue, Reader};
+use super::{BinarySource, DomainDecode, IOValue, IOValueDomainCodec, NestedValue, Reader};
 
-pub fn from_bytes<D: Embeddable, N: NestedValue<D>, Dec: DomainDecode<D>>(
+pub fn from_bytes<N: NestedValue, Dec: DomainDecode<N::Embedded>>(
     bs: &[u8],
     decode_embedded: Dec,
 ) -> io::Result<N> {
@@ -20,7 +20,7 @@ pub fn iovalue_from_bytes(bs: &[u8]) -> io::Result<IOValue> {
     from_bytes(bs, IOValueDomainCodec)
 }
 
-pub fn annotated_from_bytes<D: Embeddable, N: NestedValue<D>, Dec: DomainDecode<D>>(
+pub fn annotated_from_bytes<N: NestedValue, Dec: DomainDecode<N::Embedded>>(
     bs: &[u8],
     decode_embedded: Dec,
 ) -> io::Result<N> {
