@@ -8,9 +8,9 @@ use crate::value::reader::BytesBinarySource;
 
 use std::io;
 
-use super::{DomainParse, Embeddable, IOValue, IOValueDomainCodec, NestedValue, Reader, ViaCodec};
+use super::{DomainParse, IOValue, IOValueDomainCodec, NestedValue, Reader, ViaCodec};
 
-pub fn from_str<D: Embeddable, N: NestedValue<D>, Dec: DomainParse<D>>(
+pub fn from_str<N: NestedValue, Dec: DomainParse<N::Embedded>>(
     s: &str,
     decode_embedded: Dec,
 ) -> io::Result<N> {
@@ -21,7 +21,7 @@ pub fn iovalue_from_str(s: &str) -> io::Result<IOValue> {
     from_str(s, ViaCodec::new(IOValueDomainCodec))
 }
 
-pub fn annotated_from_str<D: Embeddable, N: NestedValue<D>, Dec: DomainParse<D>>(
+pub fn annotated_from_str<N: NestedValue, Dec: DomainParse<N::Embedded>>(
     s: &str,
     decode_embedded: Dec,
 ) -> io::Result<N> {
