@@ -50,7 +50,9 @@ fn append_codepoint(bs: &mut Vec<u8>, n: u32) -> io::Result<()> {
     Ok(())
 }
 
-impl<'de, 'src, D: Embeddable, Dec: DomainParse<D>, S: BinarySource<'de>> TextReader<'de, 'src, D, Dec, S> {
+impl<'de, 'src, D: Embeddable, Dec: DomainParse<D>, S: BinarySource<'de>>
+    TextReader<'de, 'src, D, Dec, S>
+{
     pub fn new(source: &'src mut S, dec: Dec) -> Self {
         TextReader {
             source,
@@ -87,14 +89,14 @@ impl<'de, 'src, D: Embeddable, Dec: DomainParse<D>, S: BinarySource<'de>> TextRe
     }
 
     // TODO: This is a duplicate of fn expected in PackedReader.
-    fn expected<N: NestedValue<D>>(&mut self, k: ExpectedKind) -> Error {
-        match Reader::<D, N>::demand_next(self, true) {
+    fn expected<N: NestedValue<Embedded = D>>(&mut self, k: ExpectedKind) -> Error {
+        match Reader::<N>::demand_next(self, true) {
             Ok(v) => Error::Expected(k, Received::ReceivedOtherValue(format!("{:?}", v))),
             Err(e) => e.into()
         }
     }
 
-    fn gather_annotations<N: NestedValue<D>>(&mut self) -> ReaderResult<Vec<N>> {
+    fn gather_annotations<N: NestedValue<Embedded = D>>(&mut self) -> ReaderResult<Vec<N>> {
         let mut vs = Vec::new();
         loop {
             self.skip_whitespace();
@@ -111,7 +113,7 @@ impl<'de, 'src, D: Embeddable, Dec: DomainParse<D>, S: BinarySource<'de>> TextRe
             self.skip_whitespace();
             match self.peek()? {
                 b';' => { self.skip()?; self.comment_line()?; },
-                b'@' => { self.skip()?; Reader::<D, DummyValue<D>>::skip_value(self)?; },
+                b'@' => { self.skip()?; Reader::<DummyValue<D>>::skip_value(self)?; },
                 _ => return Ok(()),
             }
         }
@@ -135,7 +137,7 @@ impl<'de, 'src, D: Embeddable, Dec: DomainParse<D>, S: BinarySource<'de>> TextRe
         }
     }
 
-    fn read_intpart<N: NestedValue<D>>(&mut self, mut bs: Vec<u8>, c: u8) -> io::Result<N> {
+    fn read_intpart<N: NestedValue>(&mut self, mut bs: Vec<u8>, c: u8) -> io::Result<N> {
         match c {
             b'0' => {
                 bs.push(c);
@@ -148,7 +150,7 @@ impl<'de, 'src, D: Embeddable, Dec: DomainParse<D>, S: BinarySource<'de>> TextRe
         }
     }
 
-    fn read_fracexp<N: NestedValue<D>>(&mut self, mut bs: Vec<u8>) -> io::Result<N> {
+    fn read_fracexp<N: NestedValue>(&mut self, mut bs: Vec<u8>) -> io::Result<N> {
         let mut is_float = false;
         match self.peek() {
             Ok(b'.') => {
@@ -168,7 +170,7 @@ impl<'de, 'src, D: Embeddable, Dec: DomainParse<D>, S: BinarySource<'de>> TextRe
         }
     }
 
-    fn read_sign_and_exp<N: NestedValue<D>>(&mut self, mut bs: Vec<u8>) -> io::Result<N> {
+    fn read_sign_and_exp<N: NestedValue>(&mut self, mut bs: Vec<u8>) -> io::Result<N> {
         match self.peek()? {
             b'+' | b'-' => bs.push(self.next_byte()?),
             _ => (),
@@ -178,7 +180,7 @@ impl<'de, 'src, D: Embeddable, Dec: DomainParse<D>, S: BinarySource<'de>> TextRe
         self.finish_number(bs, true)
     }
 
-    fn finish_number<N: NestedValue<D>>(&mut self, bs: Vec<u8>, is_float: bool) -> io::Result<N> {
+    fn finish_number<N: NestedValue>(&mut self, bs: Vec<u8>, is_float: bool) -> io::Result<N> {
         let s = decode_utf8(bs)?;
         if is_float {
             match self.peek() {
@@ -288,7 +290,7 @@ impl<'de, 'src, D: Embeddable, Dec: DomainParse<D>, S: BinarySource<'de>> TextRe
             })?)
     }
 
-    fn read_literal_binary<N: NestedValue<D>>(&mut self) -> io::Result<N> {
+    fn read_literal_binary<N: NestedValue>(&mut self) -> io::Result<N> {
         Ok(N::new(&self.read_stringlike(
             Vec::new(),
             |bs, b| Ok(bs.push(b)),
@@ -297,7 +299,7 @@ impl<'de, 'src, D: Embeddable, Dec: DomainParse<D>, S: BinarySource<'de>> TextRe
             |bs, r| Ok(bs.push(r.hexnum(2)? as u8)))?[..]))
     }
 
-    fn read_hex_binary<N: NestedValue<D>>(&mut self) -> io::Result<N> {
+    fn read_hex_binary<N: NestedValue>(&mut self) -> io::Result<N> {
         let mut s = String::new();
         loop {
             self.skip_whitespace();
@@ -315,7 +317,7 @@ impl<'de, 'src, D: Embeddable, Dec: DomainParse<D>, S: BinarySource<'de>> TextRe
         }
     }
 
-    fn read_base64_binary<N: NestedValue<D>>(&mut self) -> io::Result<N> {
+    fn read_base64_binary<N: NestedValue>(&mut self) -> io::Result<N> {
         let mut bs = Vec::new();
         loop {
             self.skip_whitespace();
@@ -332,7 +334,7 @@ impl<'de, 'src, D: Embeddable, Dec: DomainParse<D>, S: BinarySource<'de>> TextRe
         }
     }
 
-    fn upto<N: NestedValue<D>>(&mut self, delimiter: u8, read_annotations: bool) -> io::Result<Vec<N>> {
+    fn upto<N: NestedValue<Embedded = D>>(&mut self, delimiter: u8, read_annotations: bool) -> io::Result<Vec<N>> {
         let mut vs = Vec::new();
         loop {
             self.skip_whitespace();
@@ -340,11 +342,11 @@ impl<'de, 'src, D: Embeddable, Dec: DomainParse<D>, S: BinarySource<'de>> TextRe
                 self.skip()?;
                 return Ok(vs);
             }
-            vs.push(Reader::<D, N>::demand_next(self, read_annotations)?);
+            vs.push(Reader::<N>::demand_next(self, read_annotations)?);
         }
     }
 
-    fn read_dictionary<N: NestedValue<D>>(&mut self, read_annotations: bool) -> io::Result<N> {
+    fn read_dictionary<N: NestedValue<Embedded = D>>(&mut self, read_annotations: bool) -> io::Result<N> {
         let mut d = Map::new();
         loop {
             self.skip_whitespace();
@@ -352,17 +354,17 @@ impl<'de, 'src, D: Embeddable, Dec: DomainParse<D>, S: BinarySource<'de>> TextRe
                 self.skip()?;
                 return Ok(N::new(d));
             }
-            let k = Reader::<D, N>::demand_next(self, read_annotations)?;
+            let k = Reader::<N>::demand_next(self, read_annotations)?;
             self.skip_whitespace();
             if self.next_byte()? != b':' {
                 return Err(io_syntax_error("Missing expected key/value separator"));
             }
-            let v = Reader::<D, N>::demand_next(self, read_annotations)?;
+            let v = Reader::<N>::demand_next(self, read_annotations)?;
             d.insert(k, v);
         }
     }
 
-    fn read_raw_symbol<N: NestedValue<D>>(&mut self, mut bs: Vec<u8>) -> io::Result<N> {
+    fn read_raw_symbol<N: NestedValue>(&mut self, mut bs: Vec<u8>) -> io::Result<N> {
         loop {
             let c = match self.peek() {
                 Err(e) if is_eof_io_error(&e) => b' ',
@@ -383,8 +385,8 @@ impl<'de, 'src, D: Embeddable, Dec: DomainParse<D>, S: BinarySource<'de>> TextRe
     }
 }
 
-impl<'de, 'src, D: Embeddable, N: NestedValue<D>, Dec: DomainParse<D>, S: BinarySource<'de>>
-    Reader<'de, D, N> for TextReader<'de, 'src, D, Dec, S>
+impl<'de, 'src, N: NestedValue, Dec: DomainParse<N::Embedded>, S: BinarySource<'de>>
+    Reader<'de, N> for TextReader<'de, 'src, N::Embedded, Dec, S>
 {
     fn next(&mut self, read_annotations: bool) -> io::Result<Option<N>> {
         self.skip_whitespace();
@@ -415,7 +417,7 @@ impl<'de, 'src, D: Embeddable, N: NestedValue<D>, Dec: DomainParse<D>, S: Binary
                 if read_annotations {
                     let mut annotations = self.gather_annotations()?;
                     let (existing_annotations, v) =
-                        Reader::<D, N>::demand_next(self, read_annotations)?.pieces();
+                        Reader::<N>::demand_next(self, read_annotations)?.pieces();
                     annotations.extend_from_slice(existing_annotations.slice());
                     N::wrap(Annotations::new(Some(annotations)), v)
                 } else {
@@ -491,13 +493,13 @@ impl<'de, 'src, D: Embeddable, N: NestedValue<D>, Dec: DomainParse<D>, S: Binary
         if self.peek()? != b'<' { return Err(self.expected::<N>(ExpectedKind::Record(arity))); }
         self.skip()?;
         let mut b = B::Type::default();
-        Reader::<D, N>::ensure_more_expected(self, &mut b, &B::Item::RecordLabel)?;
+        Reader::<N>::ensure_more_expected(self, &mut b, &B::Item::RecordLabel)?;
         Ok(b)
     }
 
     fn open_sequence_or_set(&mut self) -> ReaderResult<B::Item> {
         self.skip_annotations()?;
-        let mark = Reader::<D, N>::mark(self)?;
+        let mark = Reader::<N>::mark(self)?;
         match self.next_byte()? {
             b'#' => match self.next_byte()? {
                 b'{' => return Ok(B::Item::SetValue),
@@ -506,7 +508,7 @@ impl<'de, 'src, D: Embeddable, N: NestedValue<D>, Dec: DomainParse<D>, S: Binary
             b'[' => return Ok(B::Item::SequenceValue),
             _ => (),
         }
-        Reader::<D, N>::restore(self, &mark)?;
+        Reader::<N>::restore(self, &mark)?;
         Err(self.expected::<N>(ExpectedKind::SequenceOrSet))
     }
 
@@ -519,7 +521,7 @@ impl<'de, 'src, D: Embeddable, N: NestedValue<D>, Dec: DomainParse<D>, S: Binary
 
     fn open_set(&mut self) -> ReaderResult<()> {
         self.skip_annotations()?;
-        let mark = Reader::<D, N>::mark(self)?;
+        let mark = Reader::<N>::mark(self)?;
         match self.next_byte()? {
             b'#' => match self.next_byte()? {
                 b'{' => return Ok(()),
@@ -527,7 +529,7 @@ impl<'de, 'src, D: Embeddable, N: NestedValue<D>, Dec: DomainParse<D>, S: Binary
             },
             _ => (),
         }
-        Reader::<D, N>::restore(self, &mark)?;
+        Reader::<N>::restore(self, &mark)?;
         Err(self.expected::<N>(ExpectedKind::Set))
     }
 
@@ -564,7 +566,7 @@ impl<'de, 'src, D: Embeddable, N: NestedValue<D>, Dec: DomainParse<D>, S: Binary
             }
             _ => {
                 b.shift(Some(i.clone()));
-                Reader::<D, N>::boundary(self, b)?;
+                Reader::<N>::boundary(self, b)?;
                 Ok(false)
             }
         }
@@ -572,7 +574,7 @@ impl<'de, 'src, D: Embeddable, N: NestedValue<D>, Dec: DomainParse<D>, S: Binary
 
     fn open_embedded(&mut self) -> ReaderResult<()> {
         self.skip_annotations()?;
-        let mark = Reader::<D, N>::mark(self)?;
+        let mark = Reader::<N>::mark(self)?;
         match self.next_byte()? {
             b'#' => match self.next_byte()? {
                 b'!' => return Ok(()),
@@ -580,7 +582,7 @@ impl<'de, 'src, D: Embeddable, N: NestedValue<D>, Dec: DomainParse<D>, S: Binary
             },
             _ => (),
         }
-        Reader::<D, N>::restore(self, &mark)?;
+        Reader::<N>::restore(self, &mark)?;
         Err(self.expected::<N>(ExpectedKind::Embedded))
     }
 
@@ -598,9 +600,9 @@ impl<'de, 'src, D: Embeddable, N: NestedValue<D>, Dec: DomainParse<D>, S: Binary
         self.source.restore(mark)
     }
 
-    fn next_token(&mut self, read_embedded_annotations: bool) -> io::Result<Token<D, N>> {
+    fn next_token(&mut self, read_embedded_annotations: bool) -> io::Result<Token<N>> {
         self.skip_annotations()?;
-        let mark = Reader::<D, N>::mark(self)?;
+        let mark = Reader::<N>::mark(self)?;
         Ok(match self.next_byte()? {
             b'<' => Token::Compound(CompoundClass::Record),
             b'[' => Token::Compound(CompoundClass::Sequence),
@@ -615,18 +617,18 @@ impl<'de, 'src, D: Embeddable, N: NestedValue<D>, Dec: DomainParse<D>, S: Binary
                 }
                 b'{' => Token::Compound(CompoundClass::Set),
                 _ => {
-                    Reader::<D, N>::restore(self, &mark)?;
+                    Reader::<N>::restore(self, &mark)?;
                     Token::Atom(self.demand_next(false)?)
                 }
             },
             _ => {
-                Reader::<D, N>::restore(self, &mark)?;
+                Reader::<N>::restore(self, &mark)?;
                 Token::Atom(self.demand_next(false)?)
             }
         })
     }
 
-    fn next_annotations_and_token(&mut self) -> io::Result<(Vec<N>, Token<D, N>)> {
+    fn next_annotations_and_token(&mut self) -> io::Result<(Vec<N>, Token<N>)> {
         let annotations = self.gather_annotations()?;
         Ok((annotations, self.next_token(true)?))
     }
